/******************************************************************************
* GeomLib.h - header file of the geometry library.			      *
*******************************************************************************
* (C) Gershon Elber, Technion, Israel Institute of Technology                 *
*******************************************************************************
* Written by Gershon Elber, June 1996.					      *
******************************************************************************/

#ifndef	GEOM_LIB_H
#define GEOM_LIB_H

#include "iritprsr.h"
#include "attribut.h"

typedef enum {
    GEOM_NO_OGL_SUPPORT,
    GEOM_OGL_NO_X_SERVER,
    GEOM_X_NO_OGL_SERVER,
    GEOM_X_NO_VISUAL,
    GEOM_X_NO_GCONTEXT,
    GEOM_CH_STACK_OVERFLOW,
    GEOM_CH_STACK_UNDERFLOW,
    GEOM_NO_INSTANCE_ORIGIN,
    GEOM_ANIM_MAT_OR_CRV,
    GEOM_UNKNOWN_ANIM_CRVS,
    GEOM_NO_ANIM_CRVS,
    GEOM_UNEQUAL_NUM_OF_POLYS,
    GEOM_UNEQUAL_NUM_OF_VRTXS,
    GEOM_TOO_MANY_ADJACENCIES,
    GEOM_NO_IRIT_PATH,
    GEOM_INVALID_FONT,

    GEOM_ERR_UNDEFINE_ERR
} GeomFatalErrorType;

/* Used by the Ray & Polygon intersection (Jordan theorem): */
#define GM_BELOW_RAY		1
#define GM_ON_RAY		2
#define GM_ABOVE_RAY		3
#define GM_ANIM_DEFAULT_FILE_NAME	"IAnim"
#define PRIM_MIN_RESOLUTION	4

#define GM_ANIM_NO_DEFAULT_TIME IRIT_INFNTY

#define GM_QUAT_COPY(SrcQ, DstQ)       GEN_COPY(DstQ, SrcQ, sizeof(GMQuatType))

typedef struct GMAnimationStruct {
    RealType
	StartT,		                     /* Starting time of animation. */
	FinalT,		                  /* Termination time of animation. */
	Dt,		                         /* Step size pf animation. */
	RunTime;		              /* Current time of animation. */
    int TwoWaysAnimation,   /* Should the animation bounce back and forth!? */
	SaveAnimation,               /* Save animation sequence in a file!? */
	BackToOrigin,	           /* Should we terminate at the beginning? */
	NumOfRepeat,			            /* How many iterations? */
	StopAnim,		   /* If TRUE, must stop the animation now. */
	SingleStep,			     /* Are we in single step mode? */
	TextInterface,		/* Are we communicating using a textual UI? */
	MiliSecSleep,	   /* How many miliseconds to sleep between frames. */
	_Count;						/* Used internally. */
    char
	*ExecEachStep,		      /* Program to execute each iteration. */
	BaseFileName[LINE_LEN];      /* Base name of animation files saved. */
} GMAnimationStruct;

typedef struct GMConvexHullPoint {
     RealType Pt[2];
} GMConvexHullPoint;

typedef struct GMBBBboxStruct {
    RealType Min[3];
    RealType Max[3];
} GMBBBboxStruct;

typedef RealType GMLsPoint[3];    /* The Z component is pretty much ignored. */

typedef struct GMLsLineSegStruct {
    struct GMLsLineSegStruct *Pnext;
    GMLsPoint Pts[2];
    long Id;			    /* Lines with unique ID never intersect. */
    VoidPtr PAux;	    /* Auxiliary backpointer - not used by ln_sweep. */
    struct GMLsIntersectStruct *Inters;
    GMLsPoint _MinVals;			         /* Bounding box on the line */
    GMLsPoint _MaxVals;
    GMLsPoint _Vec;		     /* A vector from first point to second. */
    RealType _ABC[3];			    /* Line equation as Ax + By + C. */
} GMLsLineSegStruct;

typedef struct GMLsIntersectStruct {
    struct GMLsIntersectStruct *Pnext;
    RealType t;
    RealType OtherT;
    struct GMLsLineSegStruct *OtherSeg;
    long Id;				       /* Unique ID of intersection. */
} GMLsIntersectStruct;

typedef RealType (*GMPolyOffsetAmountFuncType)(RealType *Coord);

typedef enum {             /* Predefined indices for the TransformVectorType */
    GM_QUAT_ROT_X = 0,
    GM_QUAT_ROT_Y,
    GM_QUAT_ROT_Z, 
    GM_QUAT_TRANS_X,
    GM_QUAT_TRANS_Y,
    GM_QUAT_TRANS_Z, 
    GM_QUAT_SCALE
} GMQuatTransformationsType;

typedef	RealType GMQuatType[4];                             /* A Quaternion. */
typedef RealType GMQuatTransVecType[7];        /* Transformation parameters. */

typedef RealType (* GMDistEnergy1DFuncType)(RealType);

typedef void (*GMScanConvertApplyFuncType)(int x, int y);

typedef void (*GMTransObjUpdateFuncType)(IPObjectStruct *OldPObj,
					 IPObjectStruct *NewPObj,
					 MatrixType Mat);

extern int _PrimGlblResolution;

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/* And prototypes of the functions: */

void GMVecCopy(VectorType Vdst, VectorType Vsrc);
void GMVecNormalize(VectorType V);
RealType GMVecLength(VectorType V);
void GMVecCrossProd(VectorType Vres, VectorType V1, VectorType V2);
int GMCollinear3Pts(PointType Pt1, PointType Pt2, PointType Pt3);
int GMCollinear3PtsInside(PointType Pt1, PointType Pt2, PointType Pt3);
int GMCoplanar4Pts(PointType Pt1, PointType Pt2, PointType Pt3, PointType Pt4);
RealType GMVecDotProd(VectorType V1, VectorType V2);

IPObjectStruct *GMGenMatObjectRotX(RealType *Degree);
IPObjectStruct *GMGenMatObjectRotY(RealType *Degree);
IPObjectStruct *GMGenMatObjectRotZ(RealType *Degree);
IPObjectStruct *GMGenMatObjectTrans(VectorType Vec);
IPObjectStruct *GMGenMatObjectScale(VectorType Vec);
IPObjectStruct *GMGetMatTransPortion(IPObjectStruct *MatObj, int TransPortion);
IPObjectStruct *GMGenMatObjectGeneric(IPObjectStruct *LstObjList);
GMTransObjUpdateFuncType GMTransObjSetUpdateFunc(GMTransObjUpdateFuncType
								   UpdateFunc);
IPObjectStruct *GMTransformObject(IPObjectStruct *PObj, MatrixType Mat);
IPObjectStruct *GMTransformObjectList(IPObjectStruct *PObj, MatrixType Mat);
IPObjectStruct *GMTransObjUpdateAnimCrvs(IPObjectStruct *PAnim,
					 MatrixType Mat);
IPObjectStruct *GMGenMatObjectZ2Dir(VectorType Dir);
IPObjectStruct *GMGenMatObjectZ2Dir2(VectorType Dir, VectorType Dir2);
IPObjectStruct *GMGenMatObjectRotVec(VectorType Vec, RealType *Degree);

/* General basic computational geometry routines: */

RealType GMDistPointPoint(PointType P1, PointType P2);
int GMFindLinComb2Vecs(VectorType V1,
		       VectorType V2,
		       VectorType V,
		       RealType w[2]);
int GMPlaneFrom3Points(PlaneType Plane,
		       PointType Pt1,
		       PointType Pt2,
		       PointType Pt3);
void GMPointFromPointLine(PointType Point,
			  PointType Pl,
			  PointType Vl,
			  PointType ClosestPoint);
RealType GMDistPointLine(PointType Point, PointType Pl, PointType Vl);
RealType GMDistPointPlane(PointType Point, RealType Plane[4]);
int GMPointFromLinePlane(PointType Pl,
			 PointType Vl,
			 RealType Plane[4],
			 PointType InterPoint,
			 RealType *t);
int GMPointFromLinePlane01(PointType Pl,
			   PointType Vl,
			   RealType Plane[4],
			   PointType InterPoint,
			   RealType *t);
int GM2PointsFromLineLine(PointType Pl1,
			  PointType Vl1,
			  PointType Pl2,
			  PointType Vl2,
			  PointType Pt1,
			  RealType *t1,
			  PointType Pt2,
			  RealType *t2);
RealType GMDistLineLine(PointType Pl1,
			PointType Vl1,
			PointType Pl2,
			PointType Vl2);
int GMPointFrom3Planes(PlaneType Pl1,
		       PlaneType Pl2,
		       PlaneType Pl3,
		       PointType Pt);

int GMPolygonRayInter(IPPolygonStruct *Pl, PointType PtRay, int RayAxes);
int GMPolygonRayInter3D(IPPolygonStruct *Pl, PointType PtRay, int RayAxes);
void GMGenTransMatrixZ2Dir(MatrixType Mat,
			   VectorType Trans,
			   VectorType Dir,
			   RealType Scale);
void GMGenMatrixX2Dir(MatrixType Mat, VectorType Dir);
void GMGenMatrixY2Dir(MatrixType Mat, VectorType Dir);
void GMGenMatrixZ2Dir(MatrixType Mat, VectorType Dir);
void GMGenTransMatrixZ2Dir2(MatrixType Mat,
			    VectorType Trans,
			    VectorType Dir,
			    VectorType Dir2,
			    RealType Scale);
void GMGenMatrixZ2Dir2(MatrixType Mat, VectorType Dir, VectorType Dir2);
int GMMatFromPosDir(PointType Pos,
		    VectorType Dir,
		    VectorType UpDir,
		    MatrixType M);
void GMGenMatrixRotVec(MatrixType Mat, VectorType Vec, RealType Angle);
RealType *GMBaryCentric3Pts(PointType Pt1,
			    PointType Pt2,
			    PointType Pt3,
			    PointType Pt);
int GM2PointsFromCircCirc(PointType Center1,
			  RealType Radius1,
			  PointType Center2,
			  RealType Radius2,
			  PointType Inter1,
			  PointType Inter2);
int GM2PointsFromCircCirc3D(PointType Cntr1,
			    VectorType Nrml1,
			    RealType Rad1,
			    PointType Cntr2,
			    VectorType Nrml2,
			    RealType Rad2,
			    PointType Inter1,
			    PointType Inter2);

/* Polynomial solvers. */

int GMSolveQuadraticEqn(RealType A, RealType B, RealType *Sols);
int GMSolveCubicEqn(RealType A, RealType B, RealType C, RealType *Sols);

/* Geometric properties routines: */

double GMPolyObjectArea(IPObjectStruct *PObj);
double GMPolyOnePolyArea(IPPolygonStruct *Pl);
double GMPolyObjectVolume(IPObjectStruct *PObj);
double GMPolyCountPolys(IPObjectStruct *PObj);

/* Functions from sphere's cone distribution - Sph_Cone.c. */

typedef void (* GMSphConeQueryCallBackFuncType)(IPVertexStruct *V);
typedef int (* GMSphConeQueryDirFuncType)(VectorType Vec, RealType ConeAngle);

void GMSphConeSetConeDensity(int n);
VoidPtr GMSphConeQueryInit(IPObjectStruct *PObj);
void GMSphConeQueryFree(VoidPtr SphCone);
void GMSphConeQueryGetVectors(VoidPtr SphConePtr,
			      VectorType Dir,
			      RealType Angle,
			      GMSphConeQueryCallBackFuncType SQFunc);
void GMSphConeQuery2GetVectors(VoidPtr SphConePtr,
			       GMSphConeQueryDirFuncType SQQuery,
			       GMSphConeQueryCallBackFuncType SQFunc);

/* Functions from the convex hull computation package. */

int GMConvexHull(GMConvexHullPoint *DTPts, int *NumOfPoints);

/* Functions to extract silhouette and boundary curves from polygonal data. */

VoidPtr GMSilPreprocessPolys(IPObjectStruct *PObj, int n);
int GMSilPreprocessRefine(VoidPtr PrepSils, int n);
IPObjectStruct *GMSilExtractSil(VoidPtr PrepSils, MatrixType ViewMat);
IPObjectStruct *GMSilExtractSilDirect(IPObjectStruct *PObj,
				      MatrixType ViewMat);
IPObjectStruct *GMSilExtractBndry(IPObjectStruct *PObj);
void GMSilProprocessFree(VoidPtr PrepSils);
int GMSilOrigObjAlive(int ObjAlive);

/* Functions from the animate package. */

void GMAnimResetAnimStruct(GMAnimationStruct *Anim);
void GMAnimGetAnimInfoText(GMAnimationStruct *Anim);
int GMAnimHasAnimation(IPObjectStruct *PObjs);
int GMAnimHasAnimationOne(IPObjectStruct *PObj);
void GMAnimFindAnimationTimeOne(GMAnimationStruct *Anim, IPObjectStruct *PObj);
void GMAnimFindAnimationTime(GMAnimationStruct *Anim, IPObjectStruct *PObjs);
void GMAnimSaveIterationsToFiles(GMAnimationStruct *Anim,
				 IPObjectStruct *PObjs);
RealType GMExecuteAnimationEvalMat(IPObjectStruct *AnimationP,
				   RealType Time,
				   MatrixType ObjMat);
void GMAnimDoAnimation(GMAnimationStruct *Anim, IPObjectStruct *PObjs);
int GMAnimSetAnimInternalNodes(int AnimInternalNodes);
void GMAnimEvalAnimation(RealType t, IPObjectStruct *PObj);
void GMAnimEvalAnimationList(RealType t, IPObjectStruct *PObjList);
void GMAnimDoSingleStep(GMAnimationStruct *Anim, IPObjectStruct *PObjs);
int GMAnimCheckInterrupt(GMAnimationStruct *Anim);

/* Functions from the bbox package. */

GMBBBboxStruct *GMBBComputeBboxObject(IPObjectStruct *PObj);
GMBBBboxStruct *GMBBComputeBboxObjectList(IPObjectStruct *PObj);
GMBBBboxStruct *GMBBComputeOnePolyBbox(IPPolygonStruct *PPoly);
GMBBBboxStruct *GMBBComputePolyListBbox(IPPolygonStruct *PPoly);
GMBBBboxStruct *GMBBComputePointBbox(RealType *Pt);
GMBBBboxStruct *GMBBMergeBbox(GMBBBboxStruct *Bbox1, GMBBBboxStruct *Bbox2);

/* Functions from the convex polygons package. */

int GMConvexPolyNormals(int HandleNormals);
IPObjectStruct *GMConvexPolyObjectN(IPObjectStruct *PObj);
void GMConvexPolyObject(IPObjectStruct *PObj);
int GMConvexPolygon(IPPolygonStruct *Pl);
IPPolygonStruct *GMSplitNonConvexPoly(IPPolygonStruct *Pl);
void GMGenRotateMatrix(MatrixType Mat, VectorType Dir);
void GMReverseVrtxList(IPPolygonStruct *Pl);

/* Functions from the polygonal decimation package. */

IPObjectStruct *GMDecimateObject(IPObjectStruct *);
void GMDecimateObjSetDistParam(RealType);
void GMDecimateObjSetPassNumParam(int);
void GMDecimateObjSetDcmRatioParam(int);
void GMDecimateObjSetMinAspRatioParam(RealType);

VoidPtr HDSCnvrtPObj2QTree(IPObjectStruct *PObjects, int Depth);
IPObjectStruct *HDSThreshold(VoidPtr Qt, RealType Threshold);
IPObjectStruct *HDSTriBudget(VoidPtr Qt, int TriBudget);
void HDSFreeQTree(VoidPtr Qt);
int HDSGetActiveListCount(VoidPtr Qt);
int HDSGetTriangleListCount(VoidPtr Qt);
int HDSGetDismissedTrianglesCount(VoidPtr Qt);

/* Functions from the normal interpolation package. */

void GMUpdateVerticesNormals(IPPolygonStruct *PlList,
			     IPPolygonStruct *OriginalPl);
int GMCollinear3Vertices(IPVertexStruct *V1,
			 IPVertexStruct *V2,
			 IPVertexStruct *V3);
void GMInterpNrmlBetweenTwo(IPVertexStruct *V,
			    IPVertexStruct *V1,
			    IPVertexStruct *V2);
void GMInterpNrmlBetweenTwo2(PointType Pt,
			     VectorType Normal,
			     IPVertexStruct *V1,
			     IPVertexStruct *V2);
void GMBlendNormalsToVertices(IPPolygonStruct *PlList,
			      RealType MaxAngle);

/* Functions from the line sweep package. */

void GMLineSweep(GMLsLineSegStruct **Lines);

/* Functions from the polygonal cleaning package. */

void GMCleanUpPolygonList(IPPolygonStruct **PPolygons);
void GMCleanUpPolylineList(IPPolygonStruct **PPolylines);
void GMVrtxListToCircOrLin(IPPolygonStruct *Pls, int DoCirc);
IPVertexStruct *GMFilterInteriorVertices(IPVertexStruct *VHead,
					 RealType MinTol,
					 int n);

/* Functions from the points on polygonal objects package. */

IPObjectStruct *GMConvertPolysToNGons(IPObjectStruct *PolyObj, int n);
IPObjectStruct *GMConvertPolysToTriangles(IPObjectStruct *PolyObj);
IPPolygonStruct *GMMatchPointListIntoPolylines(IPObjectStruct *PtsList,
					       RealType MaxTol);
IPObjectStruct *GMPointCoverOfPolyObj(IPObjectStruct *PolyObj,
				      int n,
				      RealType *Dir);
IPObjectStruct *GMRegularizeTriangleModel(IPObjectStruct *PObj);

/* Functions from the polygonal offsets package. */

RealType GMPolyOffsetAmountDepth(RealType *Coord);
IPPolygonStruct *GMPolyOffset(IPPolygonStruct *Poly,
			      int IsPolygon,
			      RealType Ofst,
			      GMPolyOffsetAmountFuncType AmountFunc);

/* Functions from the primitive constructions' package. */

int PrimSetPolygonalPrimitives(int PolygonalPrimitive);
int PrimSetSurfacePrimitiveRational(int SurfaceRational);

IPObjectStruct *PrimGenBOXObject(VectorType Pt,
				 RealType WidthX,
				 RealType WidthY,
				 RealType WidthZ);
IPObjectStruct *PrimGenGBOXObject(VectorType Pt,
				  VectorType Dir1,
				  VectorType Dir2,
				  VectorType Dir3);
IPObjectStruct *PrimGenCONEObject(VectorType Pt,
				  VectorType Dir,
				  RealType R,
				  int Bases);
IPObjectStruct *PrimGenCONE2Object(VectorType Pt,
				   VectorType Dir,
				   RealType R1,
				   RealType R2,
				   int Bases);
IPObjectStruct *PrimGenCYLINObject(VectorType Pt,
				   VectorType Dir,
				   RealType R,
				   int Bases);
IPObjectStruct *PrimGenSPHEREObject(VectorType Center, RealType R);
IPObjectStruct *PrimGenTORUSObject(VectorType Center,
				   VectorType Normal,
				   RealType Rmajor,
				   RealType Rminor);
IPObjectStruct *PrimGenPOLYDISKObject(VectorType N, VectorType T, RealType R);
IPObjectStruct *PrimGenPOLYGONObject(IPObjectStruct *PObjList, int IsPolyline);
IPObjectStruct *PrimGenObjectFromPolyList(IPObjectStruct *PObjList);
IPObjectStruct *PrimGenCROSSECObject(IPObjectStruct *PObj);
IPObjectStruct *PrimGenSURFREVObject(IPObjectStruct *Cross);
IPObjectStruct *PrimGenSURFREVAxisObject(IPObjectStruct *Cross,
					 VectorType Axis);
IPObjectStruct *PrimGenSURFREV2Object(IPObjectStruct *Cross,
				      RealType StartAngle,
				      RealType EndAngle);
IPObjectStruct *PrimGenSURFREV2AxisObject(IPObjectStruct *Cross,
					  RealType StartAngle,
					  RealType EndAngle,
					  VectorType Axis);
IPObjectStruct *PrimGenEXTRUDEObject(IPObjectStruct *Cross,
				     VectorType Dir,
				     int Bases);
IPObjectStruct *PrimGenRULEDObject(IPObjectStruct *Cross1,
				   IPObjectStruct *Cross2);

IPPolygonStruct *PrimGenPolygon4Vrtx(VectorType V1,
				     VectorType V2,
				     VectorType V3,
				     VectorType V4,
				     VectorType Vin,
				     IPPolygonStruct *Pnext);
IPPolygonStruct *PrimGenPolygon3Vrtx(VectorType V1,
				     VectorType V2,
				     VectorType V3,
				     VectorType Vin,
				     IPPolygonStruct *Pnext);

void PrimSetResolution(int Resolution);

/* Functions from the quaternions package. */

void GMQuatToMat(GMQuatType q, MatrixType Mat);
void GMQuatMatToQuat(MatrixType Mat, GMQuatType q);
void GMQuatRotationToQuat(RealType Xangle,
			  RealType Yangle, 
			  RealType Zangle,
			  GMQuatType q);
void GMQuatToRotation(GMQuatType q, VectorType *Angles, int *NumSolutions);
void GMQuatMul(GMQuatType q1, GMQuatType q2, GMQuatType QRes);
void GMQuatAdd(GMQuatType q1, GMQuatType q2, GMQuatType QRes);
int GMQuatIsUnitQuat(GMQuatType q);
void GMQuatNormalize(GMQuatType q);
void GMQuatInverse(GMQuatType SrcQ, GMQuatType DstQ);
void GMQuatRotateVec(VectorType OrigVec, GMQuatType RotQ, VectorType DestVec);
void GMQuatLog(GMQuatType SrcQ, VectorType DstVec);
void GMQuatExp(VectorType SrcVec, GMQuatType DstQ);
void GMQuatPow(GMQuatType MantisQ, RealType Expon, GMQuatType DstQ);
int GMQuatMatrixToAngles(MatrixType Mat, VectorType *Vec);
void GMQuatMatrixToTranslation(MatrixType Mat, VectorType Vec);
RealType GMQuatMatrixToScale(MatrixType Mat);
int GMQuatMatrixToVector(MatrixType Mat, GMQuatTransVecType TransVec);
void GMQuatVectorToMatrix(GMQuatTransVecType TransVec, MatrixType Mat);
void GMQuatVecToScaleMatrix(GMQuatTransVecType TransVec,
			    MatrixType ScaleMatrix);
void GMQuatVecToRotMatrix(GMQuatTransVecType TransVec, MatrixType RotMatrix);
void GMQuatVecToTransMatrix(GMQuatTransVecType TransVec,
			    MatrixType TransMatrix);
void GMMatrixToTransform(MatrixType Mat, 
			 VectorType S,
			 GMQuatType R,
			 VectorType T);

/* Functions from the spherical coverage package. */

IPObjectStruct *GMPointCoverOfUnitHemiSphere(RealType HoneyCombSize);

/* Functions from the software z buffer. */

int GMZBufferInit(int Width, int Height);
void GMZBufferFree(int ZbufferID);
void GMZBufferClear(int ZbufferID);
void GMZBufferClearSet(int ZbufferID, float Depth);
int GMZBufferInvert(int ZbufferID);
int GMZBufferRoberts(int ZbufferID);
int GMZBufferLaplacian(int ZbufferID);
RealType GMZBufferQueryZ(int ZbufferID, int x, int y);
void GMZBufferUpdatePt(int ZbufferID, int x, int y, RealType z);
void GMZBufferUpdateHLn(int ZbufferID,
			int x1,
			int x2,
			int y,
			RealType z1,
			RealType z2);
void GMZBufferUpdateLine(int ZbufferID,
			 int x1,
			 int y1,
			 int x2,
			 int y2,
			 RealType z1,
			 RealType z2);
void GMZBufferUpdateTri(int ZbufferID,
			int x1,
			int y1,
			RealType z1,
			int x2,
			int y2,
			RealType z2,
			int x3,
			int y3,
			RealType z3);

/* Functions from the z buffer based on Open GL package. */

int GMZBufferOGLInit(int Width,
		     int Height,
		     RealType ZMin,
		     RealType ZMax,
		     int OffScreen);
void GMZBufferOGLClear(void);
void GMZBufferOGLSetColor(int Red, int Green, int Blue);
void GMZBufferOGLMakeActive(int Id);
RealType GMZBufferOGLQueryZ(RealType x, RealType y);
void GMZBufferOGLQueryColor(RealType x,
			    RealType y,
			    int *Red,
			    int *Green,
			    int *Blue);
void GMZBufferOGLFlush(void);

/* Functions from the distribution of point on a line package. */

RealType *GMDistPoint1DWithEnergy(int N,
				  RealType XMin,
				  RealType XMax,
				  int Resolution,
				  GMDistEnergy1DFuncType EnergyFunc);

/* Metamorphosis of polygonal objects. */

IPPolygonStruct *GMPolygonalMorphosis(IPPolygonStruct *Pl1,
				      IPPolygonStruct *Pl2,
				      RealType t);

/* Scan conversion of polygons. */

void GMScanConvertTriangle(int Pt1[2],
			   int Pt2[2],
			   int Pt3[2],
			   GMScanConvertApplyFuncType ApplyFunc);

/* Text and string data sets. */

IPObjectStruct *GMMakeTextGeometry(char *Str,
				   VectorType Spacing,
				   RealType *Scaling);

/* Error handling. */

void GeomFatalError(GeomFatalErrorType ErrID);
char *GeomDescribeError(GeomFatalErrorType ErrID);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif	/* GEOMAT_3D_H */
