/******************************************************************************
* Misc_lib.h - header file for the misc. library.			      *
* This header is also the interface header to the world.		      *
*******************************************************************************
* (C) Gershon Elber, Technion, Israel Institute of Technology                 *
*******************************************************************************
* Written by Gershon Elber, Oct. 94.					      *
******************************************************************************/

#ifndef MISC_LIB_H
#define MISC_LIB_H

#include <stdio.h>
#include "irit_sm.h"

typedef enum {
    IC_BOOLEAN_TYPE = 1,
    IC_INTEGER_TYPE = 2,
    IC_REAL_TYPE = 3,
    IC_STRING_TYPE = 4
} IrtCfgDataType;

typedef enum {
    IRIT_IMAGE_RLE_TYPE,
    IRIT_IMAGE_PPM3_TYPE,
    IRIT_IMAGE_PPM6_TYPE
} IrtImgImageType;

typedef struct IritConfigStruct {
    char *VarName;
    char *SomeInfo;
    VoidPtr VarData;
    IrtCfgDataType VarType;
} IritConfigStruct;

typedef struct IrtImgPixelStruct {
    ByteType r, g, b;
} IrtImgPixelStruct;

typedef struct IritPriorQue {
    struct IritPriorQue *Right, *Left; /* Pointers to two sons of this node. */
    VoidPtr Data;			     /* Pointers to the data itself. */
} IritPriorQue;

typedef int (*IritPQCompFuncType)(VoidPtr, VoidPtr); /* Comparison function. */

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/* Basic dynamic memory allocation routines: */
#ifdef MALLOC_DEBUG_INFO
VoidPtr IritMalloc(unsigned Size, char *ObjType, char *FileName, int LineNum);
#define IritMalloc(x)	IritMalloc((x), #x, __FILE__, __LINE__)
#else
VoidPtr IritMalloc(unsigned Size);
#endif /* MALLOC_DEBUG_INFO */
void IritTestAllDynMemory(int PrintAllAlloc);
VoidPtr IritRealloc(VoidPtr p, unsigned size);
void IritFree(VoidPtr p);
void IritDebugMallocReset(void);
void IritDebugMallocSearchPtr(VoidPtr p, int Abort);
void IritDebugMallocAllocated(void);

/* Prototype of the configuration routines: */
void IritConfig(char *PrgmName, IritConfigStruct *SetUp, int NumVar);
void IritConfigPrint(IritConfigStruct *SetUp, int NumVar);

/* Get command line arguments. */
#ifdef USE_VARARGS
int GAGetArgs(int va_alist, ...);
#else
int GAGetArgs(int argc, ...);
#endif /* USE_VARARGS */
void GAPrintErrMsg(int Error);
void GAPrintHowTo(char *CtrlStr);

/* Generic 4x4 matrix routines. */
void MatGenUnitMat(MatrixType Mat);
int MatIsUnitMat(MatrixType Mat);
void MatGenMatTrans(RealType Tx, RealType Ty, RealType Tz, MatrixType Mat);
void MatGenMatUnifScale(RealType Scale, MatrixType Mat);
void MatGenMatScale(RealType Sx, RealType Sy, RealType Sz, MatrixType Mat);
void MatGenMatRotX1(RealType Teta, MatrixType Mat);
void MatGenMatRotX(RealType CosTeta, RealType SinTeta, MatrixType Mat);
void MatGenMatRotY1(RealType Teta, MatrixType Mat);
void MatGenMatRotY(RealType CosTeta, RealType SinTeta, MatrixType Mat);
void MatGenMatRotZ1(RealType Teta, MatrixType Mat);
void MatGenMatRotZ(RealType CosTeta, RealType SinTeta, MatrixType Mat);

void MatMultTwo4by4(MatrixType MatRes, MatrixType Mat1, MatrixType Mat2);
void MatAddTwo4by4(MatrixType MatRes, MatrixType Mat1, MatrixType Mat2);
void MatSubTwo4by4(MatrixType MatRes, MatrixType Mat1, MatrixType Mat2);
void MatScale4by4(MatrixType MatRes, MatrixType Mat, RealType *Scale);

void MatMultVecby4by4(VectorType VecRes, VectorType Vec, MatrixType Mat);
void MatMultPtby4by4(PointType PtRes, PointType Pt, MatrixType Mat);
void MatMultWVecby4by4(RealType VRes[4], RealType Vec[4], MatrixType Mat);

RealType MatDeterminantMatrix(MatrixType Mat);
int MatInverseMatrix(MatrixType M, MatrixType InvM);
RealType MatScaleFactorMatrix(MatrixType M);
void MatRotateFactorMatrix(MatrixType M, MatrixType RotMat);
void MatTranslateFactorMatrix(MatrixType M, VectorType Trans);

/* QR matrix factorization. */
int IritQRFactorization(RealType *A, int n, int m, RealType *Q, RealType *R);
int IritSolveUpperDiagMatrix(RealType *A, int n, RealType *b, RealType *x);
int IritSolveLowerDiagMatrix(RealType *A, int n, RealType *b, RealType *x);
int IritQRUnderdetermined(RealType *A, RealType *x, RealType *b, int m, int n);

/* An implementation of a priority que. */
void IritPQInit(IritPriorQue **PQ);
int IritPQEmpty(IritPriorQue *PQ);
void IritPQCompFunc(IritPQCompFuncType NewCompFunc);
VoidPtr IritPQFirst(IritPriorQue **PQ, int Delete);
VoidPtr IritPQInsert(IritPriorQue **PQ, VoidPtr NewItem);
VoidPtr IritPQDelete(IritPriorQue **PQ, VoidPtr NewItem);
VoidPtr IritPQFind(IritPriorQue *PQ, VoidPtr OldItem);
VoidPtr IritPQNext(IritPriorQue *PQ, VoidPtr CmpItem, VoidPtr BiggerThan);
int IritPQSize(IritPriorQue *PQ);
void IritPQPrint(IritPriorQue *PQ, void (*PrintFunc)(VoidPtr));
void IritPQFree(IritPriorQue *PQ, int FreeItems);
void IritPQFreeFunc(IritPriorQue *PQ, void (*FreeFunc)(VoidPtr));

/* Read/Write of images. */
IrtImgPixelStruct *IrtImgReadImage(char *ImageFileName, int *MaxX, int *MaxY);
IrtImgPixelStruct *IrtImgReadImage2(char *ImageFileName, int *MaxX, int *MaxY);
IrtImgImageType IrtImgWriteSetType(char *Image);
void IrtImgWriteOpenFile(char **argv, int XSize, int YSize);
void IrtImgWritePutLine(ByteType *Alpha, IrtImgPixelStruct *Pixels);
void IrtImgWriteCloseFile(void);

/* XGeneral routine - compatibility between Unix and Win95/WinNT/OS2/etc. */
char *IritStrdup(char *s);
void IritSleep(int MiliSeconds);
void IritRandomInit(long Seed);
RealType IritRandom(RealType Min, RealType Max);
RealType IritCPUTime(int Reset);
char *IritRealTimeDate(void);

#ifndef AMIGA
void movmem(VoidPtr Src, VoidPtr Dest, int Len);
#endif /* AMIGA */
char *searchpath(char *Name);

#ifdef STRICMP
int strnicmp(char *s1, char *s2, int n);
int stricmp(char *s1, char *s2);
#else
#if !(defined(__WINNT__) || defined(OS2GCC) || defined(AMIGA))
#define strnicmp(s1, s2, n) strncasecmp((s1), (s2), (n))
#define stricmp(s1, s2)     strcasecmp((s1), (s2))
#endif /* !(__WINNT__ || OS2GCC || AMIGA) */
#endif /* STRICMP */

#ifdef STRSTR
char *strstr(char *s, char *Pattern);
#endif /* STRSTR */

#ifdef GETCWD
char *getcwd(char *s, int Len);
#endif /* GETCWD */

void IritFatalError(char *Msg);
void IritWarningError(char *Msg);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif /* MISC_LIB_H */
